//--------------------------------------------------------------
// File     : stm32_ub_terminal.c
// Datum    : 11.09.2013
// Version  : 1.2
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : STM32F4
// IDE      : CooCox CoIDE 1.7.4
// GCC      : 4.7 2012q4
// Module   : je nach Verbindung [CDC, UART, UDP]
//            je nach Funktion [GPIO, ADC]
// Funktion : Terminal Verbindung zu einem PC
//
// im H-File kann eingestellt werden, welche Schnittstelle(n)
// aktiviert werden soll(en)
//--------------------------------------------------------------


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32_ub_terminal.h"


//--------------------------------------------------------------
// Globale Variabeln
//--------------------------------------------------------------
char string_buf[32];
uint32_t akt_port;   // 0=CDC, 1=UART, 2=UDP



//--------------------------------------------------------------
// interne Funktionen
//--------------------------------------------------------------
void P_Check_Kommand(char *ptr);
void P_CMD_MAIN_HELP(void);
uint8_t P_ASC2INT(uint8_t ascii);

//--------------------------------------------------------------
void UB_Terminal_Init(void)
{
  #ifdef USE_TERMINAL_CDC
    // init vom CDC
    UB_USB_CDC_Init();
  #endif

  #ifdef USE_TERMINAL_UART
    // init vom UART
    UB_Uart_Init();
  #endif

  #ifdef USE_TERMINAL_UDP
    // init vom UDP-Server
    if(UB_UDP_Server_Init()==UDP_INIT_OK) {
      // Server starten
      UB_UDP_Server_Connect();
    }
  #endif

  akt_port=0;
}

//--------------------------------------------------------------
void UB_Terminal_Do(void)
{
#ifdef USE_TERMINAL_CDC
  USB_CDC_STATUS_t check_usb;
  USB_CDC_RXSTATUS_t check_usb_rx;
#endif

#ifdef USE_TERMINAL_UART
  UART_RXSTATUS_t check_uart_rx;
#endif

#ifdef USE_TERMINAL_UDP
  UDP_RECEIVE_t check_udp_rx;
#endif


  //-----------------------------
  // Schnittstelle = USB-CDC
  //-----------------------------
  #ifdef USE_TERMINAL_CDC
    // test ob USB mit PC verbunden ist
    check_usb=UB_USB_CDC_GetStatus();
    if(check_usb==USB_CDC_CONNECTED) {
      // warte auf Kommando vom PC (ber USB)
      check_usb_rx=UB_USB_CDC_ReceiveString(string_buf);
      if(check_usb_rx==RX_READY) {
        // wenn Kommando empfangen wurde
        akt_port=0;
        P_Check_Kommand(string_buf);
      }
    }
  #endif


  //-----------------------------
  // Schnittstelle = UART
  //-----------------------------
  #ifdef USE_TERMINAL_UART
    // warte auf Kommando vom PC (ber UART)
    check_uart_rx=UB_Uart_ReceiveString(COM3,string_buf);
    if(check_uart_rx==U_RX_READY) {
      // wenn Kommando empfangen wurde
      akt_port=1;
      P_Check_Kommand(string_buf);
    }
  #endif


  //-----------------------------
  // Schnittstelle = UDP
  //-----------------------------
  #ifdef USE_TERMINAL_UDP
    // warte auf Kommando vom PC (ber UDP)
    check_udp_rx=UB_UDP_Server_Do(string_buf);
    if(check_udp_rx==UDP_RECEIVE_READY) {
      // wenn Kommando empfangen wurde
      akt_port=2;
      P_Check_Kommand(string_buf);
    }
  #endif
}


//--------------------------------------------------------------
// empfangenes Kommando auswerten
//--------------------------------------------------------------
void P_Check_Kommand(char *ptr)
{
  uint8_t len;

  len=strlen(ptr);

  if(strcmp(ptr,"HELP")==0) {
    P_CMD_MAIN_HELP();
  }
#ifdef USE_TERMINAL_GPIO
  else if((strncmp(ptr,"GPIO_OUT_xyy",9)==0) && (len==12)) {
    TERMINAL_GPIO_OUT(ptr);
  }
  else if((strncmp(ptr,"GPIO_LO_xyy",8)==0) && (len==11)) {
    TERMINAL_GPIO_LO(ptr);
  }
  else if((strncmp(ptr,"GPIO_HI_xyy",8)==0) && (len==11)) {
    TERMINAL_GPIO_HI(ptr);
  }
  else if((strncmp(ptr,"GPIO_IN_PU_xyy",11)==0) && (len==14)) {
    TERMINAL_GPIO_IN(ptr,0);
  }
  else if((strncmp(ptr,"GPIO_IN_PD_xyy",11)==0) && (len==14)) {
    TERMINAL_GPIO_IN(ptr,1);
  }
  else if((strncmp(ptr,"GPIO_IN_WO_xyy",11)==0) && (len==14)) {
    TERMINAL_GPIO_IN(ptr,2);
  }
  else if((strncmp(ptr,"GPIO_RD_xyy",8)==0) && (len==11)) {
    TERMINAL_GPIO_READ(ptr);
  }
#endif
#ifdef USE_TERMINAL_ADC
  else if((strncmp(ptr,"ADC_SET_MW_x",11)==0) && (len==12)) {
    TERMINAL_ADC_SET_MW(ptr);
  }
  else if((strncmp(ptr,"ADC_IN_xyy",7)==0) && (len==10)) {
    TERMINAL_ADC_IN(ptr);
  }
  else if((strncmp(ptr,"ADC_RD_xyy",7)==0) && (len==10)) {
    TERMINAL_ADC_READ(ptr);
  }
#endif
  else {
    // unbekannt
    TERMINAL_SEND("CMD=?");
  }
}


//--------------------------------------------------------------
// CMD : "HELP"
// Version und Hilfe zum PC zurcksenden
//--------------------------------------------------------------
void P_CMD_MAIN_HELP(void)
{
  TERMINAL_SEND("Terminal OK V:1.2");
  #ifdef USE_TERMINAL_GPIO
    TERMINAL_SEND("GPIO_OUT_xyy");
    TERMINAL_SEND("GPIO_LO_xyy");
    TERMINAL_SEND("GPIO_HI_xyy");
    TERMINAL_SEND("GPIO_IN_PU_xyy");
    TERMINAL_SEND("GPIO_IN_PD_xyy");
    TERMINAL_SEND("GPIO_IN_WO_xyy");
    TERMINAL_SEND("GPIO_RD_xyy");
  #endif

  #ifdef USE_TERMINAL_ADC
    TERMINAL_SEND("ADC_SET_MW_x");
    TERMINAL_SEND("ADC_IN_xyy");
    TERMINAL_SEND("ADC_RD_xyy");
  #endif
}



//--------------------------------------------------------------
// String zum PC senden
// je nach port ber eine der Schnittstellen
//--------------------------------------------------------------
void TERMINAL_SEND(char *ptr)
{
  #ifdef USE_TERMINAL_CDC
    if(akt_port==0) {
      // sende zum PC (ber USB)
      UB_USB_CDC_SendString(ptr,LFCR);
    }
  #endif

  #ifdef USE_TERMINAL_UART
    if(akt_port==1) {
      // sende zum PC (ber UART)
      UB_Uart_SendString(COM3,ptr,U_LFCR);
    }
  #endif

  #ifdef USE_TERMINAL_UDP
    if(akt_port==2) {
      // sende zum PC (ber UDP)
      UB_UDP_Server_SendString(ptr);
    }
  #endif
}


//--------------------------------------------------------------
// Teilstring in Integer umwandeln
//  "HALLO_10",6,2, -> 10
//--------------------------------------------------------------
uint32_t TERMINAL_STR2INT(char *ptr, uint8_t start, uint8_t len)
{
  uint32_t ret_wert=0;
  uint8_t wert;
  char *p;

  ascii_err_flag=0;

  p=ptr+start;
  if(len==1) {
    wert=P_ASC2INT(*p);
    ret_wert=wert;
  }
  else if(len==2) {
    wert=P_ASC2INT(*p);
    ret_wert=wert*10;
    p++;
    wert=P_ASC2INT(*p);
    ret_wert+=wert;
  }
  else if(len==3) {
    wert=P_ASC2INT(*p);
    ret_wert=wert*100;
    p++;
    wert=P_ASC2INT(*p);
    ret_wert+=wert*10;
    p++;
    wert=P_ASC2INT(*p);
    ret_wert+=wert;
  }
  else if(len==4) {
    wert=P_ASC2INT(*p);
    ret_wert=wert*1000;
    p++;
    wert=P_ASC2INT(*p);
    ret_wert+=wert*100;
    p++;
    wert=P_ASC2INT(*p);
    ret_wert+=wert*10;
    p++;
    wert=P_ASC2INT(*p);
    ret_wert+=wert;
  }

  return(ret_wert);
}



//--------------------------------------------------------------
// Integer in String umwandeln und ausgeben
//--------------------------------------------------------------
void TERMINAL_SEND_INT(char *ptr, uint16_t wert, uint16_t mode)
{
  char buf[20];
 
  if(mode==0)  sprintf(buf,"%s=%01d",ptr,wert);
  if(mode==1)  sprintf(buf,"%s=%04d",ptr,wert);

  if(mode>1) return;


  TERMINAL_SEND(buf);
}



//--------------------------------------------------------------
// Ascii-Zeichen in Integer umwandeln
// 0 -> 0
// 9 -> 9
// a -> 10
// f -> 15
// A -> 10
// F -> 15
// sonst -> 0 + error_flag
//--------------------------------------------------------------
uint8_t P_ASC2INT(uint8_t ascii)
{
  uint8_t ret_wert=0;

  if((ascii>=48) && (ascii<=57)) {
    ret_wert=ascii-48;
  }
  else if((ascii>=65) && (ascii<=70)) {
    ret_wert=ascii-55;
  }
  else if((ascii>=97) && (ascii<=102)) {
    ret_wert=ascii-87;
  }
  else {
    ascii_err_flag=1;
  }

  return(ret_wert);
}



